/* TMut
 * Copyright (C) 2006-2007 Philip Van Hoof <pvanhoof@gnome.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with self library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#if HAVE_CONFIG_H
#include "config.h"
#endif

#include <gtk/gtk.h>
#include <glib/gi18n-lib.h>

#include <tny-account-store.h>
#include <tny-account-store-view.h>

#include "tmut-platform-factory.h"
#include "tmut-menu-view.h"
#include "tmut-shell-window.h"
#include "tmut-shared.h"

int 
main (int argc, char **argv)
{
	GtkWidget *window;
	TnyAccountStore *account_store;
	TnyAccountStoreView *view;

	if (!g_thread_supported ()) 
		g_thread_init (NULL);

	gdk_threads_init ();

	gdk_threads_enter ();
	gtk_init (&argc, &argv);

	bindtextdomain(GETTEXT_PACKAGE, TMUT_LOCALE_DIR);

	window = GTK_WIDGET (tmut_shell_window_new ());
	gtk_window_set_title (GTK_WINDOW (window), _("TMut"));

	platfact = tmut_platform_factory_get_instance ();
	account_store = tny_platform_factory_new_account_store (platfact);

	view = TNY_ACCOUNT_STORE_VIEW (tmut_menu_view_new ());
	gtk_widget_show (GTK_WIDGET (view));

	tmut_shell_window_set_child (TMUT_SHELL_WINDOW (window), 
		TMUT_SHELL_CHILD (view), NULL);

	tny_account_store_view_set_account_store (view, account_store);

	g_signal_connect (window, "destroy", G_CALLBACK (gtk_main_quit), 0);

	gtk_widget_show (window);

	gtk_main();
	gdk_threads_leave ();

	return 0;
}

